// Programm: Radio5807.ino
// Sprache:  Arduinio C++
// Hardware: Arduino Nano with RDA 5807 Chip
// Version:  0.5 2013-12-01
// Autor:    R.Hoermann RBS Ulm
// Homepage: arduino.vom-kuhberg.de =>Projekte/RDA-Radio
// 89.20  92.60  94.50  94.70  97.60 98.40   99.00   101.90 104.40
// D3A2   D301   DE04   DE04   D3A5  D313    D3A5    D30B    D318
// SWR2   SWR1   SWR4   SWR4   SWR3  Bayern3 DasDing ???     ?? 

#include <Wire.h>               // Einbindung I2C-Library

float  f_fini= 92.60;            // Startfrequenz
int    i_fmin= 8700;            //  87.00 MHz untere Bandgrenze
int    i_fmax=10800;            // 108.00 MHz obere  Bandgrenze
int    i_ftun;                  // aktuell eingestellte Frequenz 
float  f_fbeg = 87.005;         // lower Bandlimit 
float  f_fend =108.00;          // upper Bandlimit
float  f_ftun;

int    i_key;                   // Variable fuer Tasten
String s_cmd;                   // auszufuehrendes Kommando

int    RDA5807_adrs=0x10;       // I2C-Address RDA Chip for sequential  Access
int    RDA5807_adrr=0x11;       // I2C-Address RDA Chip for random      Access
int    RDA5807_adrt=0x60;       // I2C-Address RDA Chip for TEA5767like Access
int    i_sidx=0;                // Zeiger auf das Array der Sender
int    ai_sender[15]={8920,  9070, 9260,9450,9740,9840,9890,9970,10100,10150,10300,10350,10440,10590,10690};
// String as_sender[15]={"8920","9070","S2","S3","S4","S5","S6","S7","S8","S9","S10","S11","S12","S13","S14"};
int    i_scnt=14;               // Anzahl der Sender -1 

int    i_vol=15;                // Lautstaerke(Volume)
int    i_rssi=0;                // Signal-Level

unsigned int aui_RDS[32];

boolean b_debug=false;

unsigned int    aui_RDA5807_Regdef[10] ={
                                    0x0758,  // 00 defaultid
                                    0x0000,  // 01 not used
                                    0xD009,  // 02 DHIZ,DMUTE,BASS, POWERUPENABLE,RDS
                                    0x0000,  // 03
                                    0x1400,  // 04 DE ? SOFTMUTE  
                                    0x84DF,  // 05 INT_MODE,SEEKTH=0110,????, Volume=15
                                    0x4000,  // 06 OPENMODE=01
                                    0x0000,  // 07 unused ?
                                    0x0000,  // 08 unused ?
                                    0x0000   // 09 unused ?
                                  };
unsigned int    aui_RDA5807_Reg[32];


//------------------------------------------
// // System Initialisierung
//------------------------------------------
void setup()
{
  Serial.begin(9600);                     // Serial Config
  Serial.println("Arduino FM Radio with RDA5807");
  Wire.begin();                           // Intialisierung I2C-Bus(2 Wire)
  RDA5807_Reset();
  RDA5807_PowerOn();
  RDA5807_setFreq(f_fini);
  RDA5807_Fade();
  Serial.write(">");                      // Prompt ausgeben
}

//-----------------------------------------------------
// MAIN Program Loop runs until Systemreset or Poweroff
//-----------------------------------------------------
void loop()                               
{
  if (Serial.available() > 0)             // liegt ein Tastendruck an
  { keyhandle(); }                        //   dann verarbeiten
}

//------------------------------------------
// Serial Print 16Bit Number in HEX as hh:ll
//------------------------------------------
void Serial_print16h(unsigned int ui_val)
{
  byte b_high,b_low;
  b_high=ui_val>>8; b_low=ui_val & 0xFF;
  if (b_high<0x10){ Serial.write('0');} Serial.print(b_high,HEX); Serial.write(':');
  if (b_low <0x10){ Serial.write('0');} Serial.print(b_low ,HEX); 
}

//----------------------------------------
// SerialPrint 16Bit Number in HEX as hhll
//----------------------------------------
void Serial_printuih(unsigned int val)
{
  if (val<  0xF) Serial.write('0');                 // if less 2 Digit
  if (val< 0xFF) Serial.write('0');                 // if less 3 Digit
  if (val<0xFFF) Serial.write('0');                 // if less 4 Digit
  Serial.print(val,HEX);
  Serial.write(' ');
}

//----------------------------------------
// Serial Set Cursor to X,Y Row/Col
//----------------------------------------
void Serial_xy(int x,int y)
{
  Serial.write(0x1B); Serial.write('['); 
  Serial.print(y); Serial.write(';'); Serial.print(x); Serial.write('H');
}

//----------------------------------------
// Serial Clear Screen
//----------------------------------------
void Serial_Cls()
{
  Serial.print("\x1B[0m\x1B[H\x1B[2J");
  //for(int i=0;i<25;i++){ Serial.println();}
}
//----------------------------------------
// Write 16Bit To I2C / Two Wire Interface
//----------------------------------------
void Wire_write16(unsigned int val)
{
  if (b_debug) { Serial_print16h(val);}
  Wire.write(val >> 8); Wire.write(val & 0xFF);
}

//------------------------------------------
// Scan I2C / Two Wire Bus for aktiv devices
//------------------------------------------
int wire_Scan()
{
  int i_devadr;
  int i_res;
  int i_ret=1;
  for (i_devadr=8;i_devadr<127;i_devadr++)
  {
    Wire.beginTransmission(i_devadr);
    i_res=Wire.endTransmission();
    if (i_res==0) 
    { 
      if (i_devadr< 16){Serial.write('0');}
      Serial.print(i_devadr,HEX);
      Serial.println("h OKAY");
      i_ret=0;
    }
  }
  return i_ret;
}
//----------------------------------------
// RDA5807 Set all Configuration Registers
//----------------------------------------
int RDA5807_Write()
{
  Wire.beginTransmission(RDA5807_adrs);
  for ( int i=2;i<7;i++){ Wire_write16(aui_RDA5807_Reg[i]);}
  return Wire.endTransmission();
}

//----------------------------------------
// RDA5807 Set one Configuration Registers
//----------------------------------------
int RDA5807_WriteReg(int i_reg)
{
  Wire.beginTransmission(RDA5807_adrr);
  Wire.write(i_reg); Wire_write16(aui_RDA5807_Reg[i_reg]);
  return Wire.endTransmission();
}

//---------------------------------------------
// RDA5807 Read Special Data Registers as Word
//---------------------------------------------
void RDA5807_ReadW(int i_cnt)
{
   Wire.beginTransmission(RDA5807_adrr);            // Device 0x11 for random access
   Wire.write(0x0C);                                // Start at Register 0x0C
   Wire.endTransmission(0);                         // restart condition
   Wire.requestFrom(RDA5807_adrr,2*i_cnt, 1);       // Retransmit device address with READ, followed by 8 bytes
   for (int i=0; i<i_cnt; i++)                      // Loop for Read data    
   {aui_RDS[i]=256*Wire.read()+Wire.read();}        // Read Data into Array of Unsigned Ints
   Wire.endTransmission();                  
} 

//--------------------------------------------
// RDA5807 Reset Chip to Default Configuration
//--------------------------------------------
int RDA5807_Reset()
{
  Serial.println("RESET RDA5807");
  for(int i=0;i<7;i++){aui_RDA5807_Reg[i]=aui_RDA5807_Regdef[i];}
  aui_RDA5807_Reg[2]=aui_RDA5807_Reg[2] | 0x0002;   // Enable SoftReset
  int i_ret=RDA5807_Write();
  aui_RDA5807_Reg[2]=aui_RDA5807_Reg[2] & 0xFFFB;   // Disable SoftReset
  return i_ret;
}

//----------------------------------------
// RDA5807 Power Off
//----------------------------------------
int RDA5807_PowerOff()
{
  RDA5807_setVol(0);
  Serial.println("Power off");
  aui_RDA5807_Reg[2]=0x0001;   // all bits off
  return RDA5807_Write();
  aui_RDA5807_Reg[2]=aui_RDA5807_Regdef[2];       // Reset to Default Value
}

//----------------------------------------
// RDA5807 Power On
//----------------------------------------
int RDA5807_PowerOn()
{
  Serial.println("Power ON");
  aui_RDA5807_Reg[3]=aui_RDA5807_Reg[3] | 0x010;   // Enable Tuning
  aui_RDA5807_Reg[2]=aui_RDA5807_Reg[2] | 0x001;   // Enable PowerOn
  int i_ret=RDA5807_Write();
  aui_RDA5807_Reg[3]=aui_RDA5807_Reg[3] & 0xFFEF;  // Disable Tuning
  return i_ret;
}

//----------------------------------------
// RDA5807 Softmute
//----------------------------------------
int RDA5807_Softmute()
{
  Serial.print("Mute ");
  if ((aui_RDA5807_Reg[4] & 0x0200)==0)
  {aui_RDA5807_Reg[4]=aui_RDA5807_Reg[4] | 0x0200; Serial.println("on ");}
  else
  {aui_RDA5807_Reg[4]=aui_RDA5807_Reg[4] & 0xFDFF; Serial.println("off");}
  return RDA5807_Write();
}

//----------------------------------------
// RDA5807 Set Volume
//----------------------------------------
int RDA5807_setVol(int i_setvol)
{
  i_vol=i_setvol;
  if (i_vol>15) {i_vol=15; Serial.println("Vol already maximal"); return 1; }
  if (i_vol <0) {i_vol= 0; Serial.println("Vol already minimal"); return 1; }
  Serial.print("Volume="); Serial.println(i_vol);
  aui_RDA5807_Reg[5]=(aui_RDA5807_Reg[5] & 0xFFF0)| i_vol;   // Set New Volume
  RDA5807_WriteReg(5);
  return 0;
}

//----------------------------------------
// RDA5807 Increment Volume
//----------------------------------------
int RDA5807_VolUp()
{ return RDA5807_setVol(++i_vol); }

//----------------------------------------
// RDA5807 Decrement Volume
//----------------------------------------
int RDA5807_VolDown()
{ return RDA5807_setVol(--i_vol); }

//----------------------------------------
// RDA5807 Maximal Volume
//----------------------------------------
int RDA5807_VolMax()
{ return RDA5807_setVol(15); }

//----------------------------------------
// RDA5807 Minimal Volume
//----------------------------------------
int RDA5807_VolMin()
{ return RDA5807_setVol(0);  }

//----------------------------------------
// RDA5807 Fade Volume
//----------------------------------------
int RDA5807_Fade()
{
  Serial.print("Fade Vol = ");
  for (int i=0;i<=15;i++)
  {
    aui_RDA5807_Reg[5]=(aui_RDA5807_Reg[5] &0xFFF0)+i;
    if (i<10){Serial.write(' ');} Serial.print(i); Serial.write(' ');
    RDA5807_WriteReg(5);
    delay(100);
  }
  Serial.println();
  Serial.print("Fade Vol = ");
  for (int i=14;i>=8;i--)
  {
    aui_RDA5807_Reg[5]=(aui_RDA5807_Reg[5] &0xFFF0)+i;
    if(i<10){Serial.write(' ');}  Serial.print(i); Serial.write(' ');
    RDA5807_WriteReg(5);
    delay(100); 
  }
  Serial.println();
  i_vol=8;
  return 0;
}

//----------------------------------------
// RDA5807 Seek up  to next Station
//----------------------------------------
int RDA5807_SeekUp()
{
  Serial.println("SeekUp");
  aui_RDA5807_Reg[2]=aui_RDA5807_Reg[2] | 0x0300;   // Enable Seekup
  RDA5807_Write();
  aui_RDA5807_Reg[2]=aui_RDA5807_Reg[2] & 0xFCFF;   // Disable Seekup
  return 0;
}

//----------------------------------------
// RDA5807 Seek down  to next Station
//----------------------------------------
int RDA5807_SeekDown()
{
  Serial.println("SeekDown");
  aui_RDA5807_Reg[2]=aui_RDA5807_Reg[2] | 0x0100;   // Enable SeekDown(default)
  RDA5807_Write();
  aui_RDA5807_Reg[2]=aui_RDA5807_Reg[2] & 0xFCFF;   // Disable Seek 
  return 0;
}

//----------------------------------------
// RDA5807 Tune Radio to defined Frequency
//----------------------------------------
int RDA5807_setFreq(float f_mhz)
{
  i_ftun=f_mhz*100; 
  f_ftun=f_mhz;
  int i_chnr=(f_mhz-f_fbeg)/0.1;
  i_chnr=i_chnr & 0x03FF;
  aui_RDA5807_Reg[3]=i_chnr*64 +0x10;  // Channel + TUNE-Bit + Band=00(87-108) + Space=00(100kHz)
  Wire.beginTransmission(RDA5807_adrs);
  Wire_write16(0xD009);
  Wire_write16(aui_RDA5807_Reg[3]);
  Wire.endTransmission(); 
  delay(100);
  RDA5807_Status();
  return 0;
}

//----------------------------------------
// RDA5807 Radio Data System Dump Infos
//----------------------------------------
int RDA5807_RDS_Dump()
{
  Serial_Cls();
  RDA5807_Status();
  Serial.println(" PI |GTxx|Asci");
  int cnt=0;
  while(Serial.available()==0)
  {
    RDA5807_ReadW(4);                              // Read RDS-Data as 4 Word to Array
    if((aui_RDS[1] & 0xF000)==0x2000)
    { 
      Serial_xy(1,4); 
      Serial_printuih(aui_RDS[0]);                 // Block A  PI
      Serial_printuih(aui_RDS[1]);                 // Block B  GT(5Bit)T(1Bit) PTY(5Bit)POS(5)Bit
      Serial_printuih(aui_RDS[2]);
      Serial_printuih(aui_RDS[3]);
      int x=16+4*(aui_RDS[1] & 0x000F);
      Serial_xy(x,7);
     
      for (int i=2;i<4;i++)  
      { 
        Serial.write(aui_RDS[i]>>8);               // Block C/D Ascii Code
        Serial.write(aui_RDS[i]&0xFF);             // 2 * 2 Byte
      }
     cnt++;
    }
    if ((aui_RDS[1] & 0xF000)==0x4000)
    {
      int i_hh =(16*(aui_RDS[2] & 0x0001)+((aui_RDS[3] & 0xF000)>>12));
      int i_mm =(aui_RDS[3] & 0x0FC0)>>6;
      int i_ofs=(aui_RDS[3] & 0x003F);
      i_hh=i_hh+(i_ofs/2);
      Serial_xy(16,5);
      if (i_hh <10){Serial.write(' ');} Serial.print(i_hh);  Serial.write(':');
      if (i_mm <10){Serial.write('0');} Serial.print(i_mm);  Serial.write(' ');
    }
   // if ((aui_RDS[1]& 0xF000)==0x400)
   // { Serial.print("RDS CT: "); for (int i=0;i<4;i++){ Serial_print16h(aui_RDS[i]); Serial.write(' | ');}  Serial.println();}
   delay(80);
   //if(cnt==20){cnt=0; Serial.write(0x1B); Serial.write('['); Serial.print(20); Serial.write('A'); } // Ansi CursorUp
  }
  for(int i=0;i<24;i++){Serial.println();}
  return  0;
}
 
//----------------------------------------
// RDA5807 Radio Data System Dump Infos
//----------------------------------------
int RDA5807_RDS_DumpCT()
{
  int          i_gt,i_gab,i_pty,i_t,i_pos,i_hh,i_mm,i_ofs;
  RDA5807_Status();
  Serial.println(" PI |GTxx|Asci      GT  T PTY POS HH:mm Offset");
  while(Serial.available()==0)
  {
    RDA5807_ReadW(4);                              // Read RDS-Data as 4 Word to Array
    i_gt = aui_RDS[1] >>12;
    if ((aui_RDS[1] & 0x0800)==0){i_gab='A';} else {i_gab='B';}
    i_t  =(aui_RDS[1] & 0x0400)>10;
    i_pty=(aui_RDS[1] & 0x03FF)>>5;
    i_pos=(aui_RDS[1] & 0x000F);
    i_hh =(16*(aui_RDS[2] & 0x0001)+((aui_RDS[3] & 0xF000)>>12));
    i_mm =(aui_RDS[3] & 0x0FC0)>>6;
    i_ofs=(aui_RDS[3] & 0x003F);
    i_hh=i_hh+(i_ofs/2);
    if (i_gt==4)
    {
    Serial_printuih(aui_RDS[0]);       // Block A  PI
    Serial_printuih(aui_RDS[1]);       // Block B  GT(4Bit) A/B(1Bit) T(1Bit) PTY(5Bit)POS(5)Bit
    Serial_printuih(aui_RDS[2]);
    Serial_printuih(aui_RDS[3]);
    if (i_gt <10){Serial.write(' ');} Serial.print(i_gt); Serial.write(i_gab); Serial.write(' ');
    if (i_t  <10){Serial.write(' ');} Serial.print(i_t);   Serial.write(' ');
    if (i_pty<10){Serial.write(' ');} Serial.print(i_pty); Serial.print("  ");
    if (i_pos<10){Serial.write(' ');} Serial.print(i_pos); Serial.write(" ");
    if (i_hh <10){Serial.write(' ');} Serial.print(i_hh);  Serial.write(':');
    if (i_mm <10){Serial.write('0');} Serial.print(i_mm);  Serial.write(' ');
    Serial.print(i_ofs);
    Serial.println();
    }
    delay(80);
  }
  return  0;
}
//----------------------------------------
// RDA5807 Show Status
//----------------------------------------
void RDA5807_Status()
{
  int i_chnr;
  Wire.requestFrom (RDA5807_adrs, 16); 
  for (int i=0; i<8; i++) { aui_RDA5807_Reg[0xA+i] = 256*Wire.read ()+Wire.read(); }
  Wire.endTransmission();
  i_chnr=aui_RDA5807_Reg[0x0A] & 0x03FF;
  i_rssi=aui_RDA5807_Reg[0x0B]>>10;
  f_ftun=f_fbeg+i_chnr*0.1;
  Serial.print("R03=");
  Serial_print16h(aui_RDA5807_Reg[3]);
  Serial.print(" ");
  if (f_ftun<=99.99){Serial.write(' ');}
  Serial.print(f_ftun);
  Serial.print(" MHz");
  Serial.print(" Ch="); if (i_chnr<10){Serial.write(' ');} if (i_chnr<100){Serial.write(' ');} Serial.print(i_chnr);
  Serial.print(" PI="); Serial_printuih(aui_RDA5807_Reg[0x0C]); // RDS Block A contains Station ID
  if ((aui_RDA5807_Reg[0x0A] & 0x0400)==0){Serial.print(" Mono  ");} else {Serial.print(" Stereo");}
  if ((aui_RDA5807_Reg[0x0A] & 0x8000)==0){Serial.print(" ---");}    else {Serial.print(" RDS");   }
  Serial.print(" Sig="); if (i_rssi<10){Serial.write(' ');} Serial.print(i_rssi);
  Serial.write(' ');
  for(int i=0;i<i_rssi-5;i++){Serial.write('*');}
  Serial.println();
}

//----------------------------------------
// RDA5807 Report all available Stations
//----------------------------------------
int RDA5807_Report()
{
  f_ftun=f_fbeg;
  int i_cnt_stations=0;
  int i_cnt_stereo=0;
  int i_cnt_rds=0;
  float f_rssi=0;
  Serial.println("Sender Report:");
  for(int i_chnr=0;i_chnr<=210;i_chnr++)
  {
    aui_RDA5807_Reg[3]=64*i_chnr+0x10; 
    Wire.beginTransmission(RDA5807_adrs);
    Wire_write16(0xD009);
    Wire_write16(aui_RDA5807_Reg[3]);
    Wire.endTransmission();
    delay(100);
    RDA5807_Status();
    if (i_rssi>7){i_cnt_stations++;}
    if ((aui_RDA5807_Reg[0x0A] & 0x0400)!=0){i_cnt_stereo++;}
    if ((aui_RDA5807_Reg[0x0A] & 0x8000)!=0){i_cnt_rds++;   }
    f_rssi=f_rssi+i_rssi;
  }
  f_rssi=f_rssi/i_cnt_stations;
  Serial.println();
  Serial.print("Found Stations: "); Serial.println(i_cnt_stations); 
  Serial.print("   with Stereo: "); Serial.println(i_cnt_stereo);
  Serial.print("      with RDS: "); Serial.println(i_cnt_rds);
  Serial.print("   Siglevelavg: "); Serial.println(f_rssi);
  Serial.println();
  RDA5807_setFreq(f_fini);
  RDA5807_Fade();
  return 0;
}

//-----------------------------------------------
// RDA5807 Calc Frequenz from Channel + get Name
//-----------------------------------------------
int RDA5807_ChanToFreq(int i_ch)
{
  float f_fmhz=f_fbeg+i_ch*0.10;
  Serial.print(i_ch);
  Serial.write("=");
  Serial.print(f_fmhz); 
  Serial.write(" MHz ");
  switch(i_ch)
  {
    case  21: Serial.write("SWR 2 Ulm");           break;
    case  22: Serial.write("SWR 2 Ulm");           break;
    case  56: Serial.write("SWR 1 ");              break;
    case  75: Serial.write("SWR 4 Schwabenradio"); break;
    case 113: Serial.write("Bayern 3");            break;
    case 199: Serial.write("Bayern 5");            break; 
    default:  Serial.write("unknown");             break;
  }
  Serial.println();
  return 0;
}

//----------------------------------------
// RDA5807 Dump all Config-Registers
//----------------------------------------
int RDA5807_Cfg_Dump()
{
  for (int i_reg=0; i_reg<8;i_reg++)
  {
    Serial.write("Reg 0"); Serial.print(i_reg,HEX); Serial.write(" = ");
    Serial_print16h(aui_RDA5807_Reg[i_reg]);
    Serial.println();
  }
  return 0;
}

//----------------------------------------
// RDA5807 Read and Show all Status Registers
//----------------------------------------
int RDA5807_ReadStatus()
{
  int i_channr=-1;
  unsigned int aui_buf[8];
  memset (aui_buf, 0, 8);
  Serial.println("Info Status RDA5807:");
  Serial.println("Reg | 0Ah | 0Bh | 0Ch | 0Dh | 0Eh | 0Fh |");
  Serial.print  ("    |");
  Wire.requestFrom(RDA5807_adrs, 12); 
  for (int i=0; i<6; i++) { aui_buf[i] = 256*Wire.read ()+Wire.read(); }
  Wire.endTransmission();
  for (int i=0; i<6;  i++) 
  { Serial_print16h(aui_buf[i]); Serial.write('|'); }
  Serial.println();
  Serial.print("RDS Data:    "); if ((aui_buf[0] & 0x8000)==0){Serial.println("NO");} else {Serial.println("NEW data");}
  Serial.print("RDS Sync:    "); if ((aui_buf[0] & 0x1000)==0){Serial.println("no");} else {Serial.println("OK");}
  Serial.print("RDS Block:   "); if ((aui_buf[0] & 0x0800)==0){Serial.println("no");} else {Serial.println("Block E");}
  Serial.print("SEEK Ready:  "); if ((aui_buf[0] & 0x4000)==0){Serial.println("no");} else {Serial.println("OK");}
  Serial.print("SEEK Fail:   "); if ((aui_buf[0] & 0x2000)==0){Serial.println("no, Succces!");} else {Serial.println("FAILED");}
  Serial.print("Stationmode: "); if ((aui_buf[0] & 0x0400)==0){Serial.println("Mono  ");} else {Serial.println("Stereo");}
  Serial.print("Channel Nr:  "); i_channr=(aui_buf[0] & 0x03FF); RDA5807_ChanToFreq(i_channr);
  Serial.print("SignalLevel: "); i_rssi=aui_buf[1]>>10; Serial.println(i_rssi);
  return 0;
}

//----------------------------------------
// Get and Interpret all Serial Keys 
//----------------------------------------
void keyhandle()
{
  i_key = Serial.read();                  // get incoming byte
  if (i_key==8 or i_key==127) 
  {
    int i_cl=s_cmd.length()-1;
    s_cmd=s_cmd.substring(0,i_cl); 
    Serial.write(8);Serial.write(32); Serial.write(8); i_key=0;
  }
  if (i_key >64 and i_key<90){i_key+=32;}  // GROSS in klein-Buchstaben
  if (i_key>=32){s_cmd=s_cmd+char(i_key); Serial.write(i_key);}
  if (i_key==13){CLI_exec();  }
}

//----------------------------------------
// Show a Command-Help on Serial Terminal 
//----------------------------------------
int RDA5807_Help()
{
  
  Serial.println("v+/v-   Volume: Up  / Down");
  Serial.println("v++/v-- Volume: Max /Min");
  Serial.println("vf      Volume: Fade");
  Serial.println("bb      Bass-Booster");
  Serial.println("s+/s-   ScanUp   / ScanDown");
  Serial.println("n+/n-   Next/Prev FixStation");
  Serial.println("swr4 swr2 br3 br5 tune to Station");
  Serial.println("m      Mute ON /OFF");
  Serial.println("f      Fade");
  Serial.println("on     Power ON");
  Serial.println("off    Power OFF");
  Serial.println("r      RESET");
  Serial.println("rds    RDS-Info Dump");
  Serial.println("rep    Report");
  Serial.println("i      Info Status");
  Serial.println("d      Dump Config");
  Serial.println("a      Dump ALL");
  Serial.println("2      i2c Scan");
  Serial.println("?      HELP");
  return 0;
}

//----------------------------------------
// Execute Command-Line Commands on ENTER 
//----------------------------------------
int CLI_exec()
{
  int i_ret=-1;
  Serial.println();
 // Serial.print("EXEC "); Serial.println(s_cmd);
  if(s_cmd=="2")   {i_ret=wire_Scan();            }

  if(s_cmd=="br3") {i_ret=RDA5807_setFreq( 98.40);}
  if(s_cmd=="ding"){i_ret=RDA5807_setFreq( 99.00);}
  if(s_cmd=="br5") {i_ret=RDA5807_setFreq(106.90);}
  if(s_cmd=="swr1"){i_ret=RDA5807_setFreq( 89.60);} 
  if(s_cmd=="swr2"){i_ret=RDA5807_setFreq( 89.20);}
  if(s_cmd=="swr3"){i_ret=RDA5807_setFreq( 97.60);}
  if(s_cmd=="swr4"){i_ret=RDA5807_setFreq( 92.60);}
    
  if(s_cmd=="s+")  {i_ret=RDA5807_SeekUp();      }
  if(s_cmd=="s-")  {i_ret=RDA5807_SeekDown();    }
  if(s_cmd=="n+")  {i_sidx++; if(i_sidx>i_scnt) {i_sidx=0;} i_ret=RDA5807_setFreq(ai_sender[i_sidx]/100.0); }
  if(s_cmd=="n-")  {i_sidx--; if(i_sidx<0) {i_sidx=i_scnt;} i_ret=RDA5807_setFreq(ai_sender[i_sidx]/100.0); }
  if(s_cmd=="rep") {i_ret=RDA5807_Report();      }
  if(s_cmd=="v+")  {i_ret=RDA5807_VolUp();       }
  if(s_cmd=="v-")  {i_ret=RDA5807_VolDown();     }
  if(s_cmd=="v--") {i_ret=RDA5807_VolMin();      }
  if(s_cmd=="v++") {i_ret=RDA5807_VolMax();      }
  if(s_cmd=="vf")  {i_ret=RDA5807_Fade();        }
  if(s_cmd=="m")   {i_ret=RDA5807_Softmute();    }
  if(s_cmd=="on")  {i_ret=RDA5807_PowerOn();     }
  if(s_cmd=="off") {i_ret=RDA5807_PowerOff();    }
  if(s_cmd=="r")   {i_ret=RDA5807_Reset();       }
  if(s_cmd=="rds") {i_ret=RDA5807_RDS_Dump();    }
  if(s_cmd=="ct")  {i_ret=RDA5807_RDS_DumpCT();   }
  if(s_cmd=="i")   {i_ret=RDA5807_ReadStatus();  }
  if(s_cmd=="d")   {i_ret=RDA5807_Cfg_Dump();    }
  if(s_cmd=="?")   {i_ret=RDA5807_Help();        }
  if(i_ret==-1)
  {Serial.print("I dont know howto "); Serial.println(s_cmd);}
  else
  {
    if(i_ret==1) {Serial.print("Error executing: ");Serial.println(s_cmd);}
  }  
  Serial.print(">"); s_cmd=""; 
  return i_ret;
}

